package com.huimv.farm.musk.breed.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedBatch;
import com.huimv.farm.musk.breed.entity.BreedFeed;
import com.huimv.farm.musk.breed.entity.BreedMuskInfo;
import com.huimv.farm.musk.breed.mapper.BreedBatchMapper;
import com.huimv.farm.musk.breed.mapper.BreedMuskInfoMapper;
import com.huimv.farm.musk.breed.service.IBreedBatchService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 编组管理 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-19
 */
@Service
public class BreedBatchServiceImpl extends ServiceImpl<BreedBatchMapper, BreedBatch> implements IBreedBatchService {

    @Autowired
    private BreedBatchMapper batchMapper;
    @Autowired
    private BreedMuskInfoMapper muskInfoMapper;
    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String batchName = paramsMap.get("batchName");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(batchName), "batch_name", batchName);
        Page page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, batchMapper.selectPage(page, queryWrapper));
    }

    @Transactional
    @Override
    public Result add(BreedBatch breedBatch) {
        QueryWrapper<BreedBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", breedBatch.getFarmId()).eq("batch_name", breedBatch.getBatchName());
        Integer count = batchMapper.selectCount(queryWrapper);
        if (count != 0) {
            return new Result(10001, "添加失败！编组名称已存在！", false);
        }
        breedBatch.setCreateTime(new Date());
        batchMapper.insert(breedBatch);
        BreedBatch batch = batchMapper.selectOne(new QueryWrapper<BreedBatch>()
                .eq("farm_id", breedBatch.getFarmId()).eq("batch_name", breedBatch.getBatchName()));
        String batchBreedMusk = breedBatch.getBreedMusk();
        String[] split = batchBreedMusk.split(",");
        for (String muskId : split) {
            BreedMuskInfo muskInfo = muskInfoMapper.selectOne(new QueryWrapper<BreedMuskInfo>().eq("musk_id", muskId));
            muskInfo.setBatchNum(batch.getId());
            muskInfo.setBatchName(batch.getBatchName());
            muskInfo.setIsBatch(1);
            muskInfoMapper.updateById(muskInfo);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Transactional
    @Override
    public Result edit(BreedBatch breedBatch) {
        Integer count = batchMapper.selectCount(new QueryWrapper<BreedBatch>().eq("farm_id", breedBatch.getFarmId())
                .eq("batch_name", breedBatch.getBatchName()).ne("id", breedBatch.getId()));
        if (count != 0) {
            return new Result(10001, "修改失败！编组名称已存在！", false);
        }
        String batchBreedMusk = breedBatch.getBreedMusk();
        String[] split = batchBreedMusk.split(",");
        for (String muskId : split) {
            BreedMuskInfo muskInfo = muskInfoMapper.selectOne(new QueryWrapper<BreedMuskInfo>().eq("musk_id", muskId));
            muskInfo.setBatchNum(breedBatch.getId());
            muskInfo.setBatchName(breedBatch.getBatchName());
            muskInfo.setIsBatch(1);
            muskInfoMapper.updateById(muskInfo);
        }
        batchMapper.updateById(breedBatch);
        return new Result(ResultCode.SUCCESS);
    }

    @Transactional
    @Override
    public Result remove(Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        BreedBatch breedBatch = batchMapper.selectById(id);
        String batchBreedMusk = breedBatch.getBreedMusk();
        String[] split = batchBreedMusk.split(",");
        for (String muskId : split) {
            BreedMuskInfo muskInfo = muskInfoMapper.selectOne(new QueryWrapper<BreedMuskInfo>().eq("musk_id", muskId));
            muskInfo.setBatchNum(null);
            muskInfo.setBatchName(null);
            muskInfo.setIsBatch(0);
            muskInfoMapper.updateById(muskInfo);
        }
        batchMapper.deleteById(id);
        return new Result(ResultCode.SUCCESS);
    }
}
