package com.huimv.farm.musk.breed.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedBatch;
import com.huimv.farm.musk.breed.entity.BreedFeed;
import com.huimv.farm.musk.breed.entity.BreedTransfer;
import com.huimv.farm.musk.breed.mapper.BreedBatchMapper;
import com.huimv.farm.musk.breed.mapper.BreedFeedMapper;
import com.huimv.farm.musk.breed.service.IBreedFeedService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.vo.DataVo;
import com.huimv.farm.musk.entity.vo.FeedDataVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.util.*;

/**
 * <p>
 * 采食计量 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedFeedServiceImpl extends ServiceImpl<BreedFeedMapper, BreedFeed> implements IBreedFeedService {

    @Autowired
    private BreedFeedMapper feedMapper;
    @Autowired
    private BreedBatchMapper batchMapper;
    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String muskId = paramsMap.get("muskId");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedFeed> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(muskId), "musk_id", muskId);
        Page page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, feedMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(BreedFeed breedFeed) {
        Integer isBatch = breedFeed.getIsBatch();
        if (isBatch == 1) {
            QueryWrapper<BreedBatch> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", breedFeed.getFarmId()).eq("id", breedFeed.getMuskId());
            BreedBatch batch = batchMapper.selectOne(queryWrapper);
            String breedMusk = batch.getBreedMusk();
            String[] split = breedMusk.split(",");
            Double dry = Double.valueOf(breedFeed.getDryLeaf());
            Double wet = Double.valueOf(breedFeed.getWetLeaf());
            Double vegetable = Double.valueOf(breedFeed.getVegetable());
            Double feed = Double.valueOf(breedFeed.getFeed());
            DecimalFormat def = new DecimalFormat();
            double d = dry / 4;
            double w = wet / 4;
            double v = vegetable / 4;
            double f = feed / 4;
            for (String muskId : split) {
                BreedFeed breedFeed1 = new BreedFeed();
                breedFeed1.setDryLeaf(def.format(d));
                breedFeed1.setWetLeaf(def.format(w));
                breedFeed1.setVegetable(def.format(v));
                breedFeed1.setFeed(def.format(f));
                breedFeed1.setMuskId(muskId);
                breedFeed1.setCreateTime(new Date());
                breedFeed1.setFarmId(breedFeed.getFarmId());
                feedMapper.insert(breedFeed1);
            }
        } else {
            BreedFeed breedFeed1 = new BreedFeed();
            breedFeed1.setMuskId(breedFeed.getMuskId());
            breedFeed1.setDryLeaf(breedFeed.getDryLeaf());
            breedFeed1.setWetLeaf(breedFeed.getWetLeaf());
            breedFeed1.setVegetable(breedFeed.getVegetable());
            breedFeed1.setFeed(breedFeed.getFeed());
            breedFeed1.setFarmId(breedFeed.getFarmId());
            breedFeed.setCreateTime(new Date());
            feedMapper.insert(breedFeed1);
        }

        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result edit(BreedFeed breedFeed) {
        return new Result(ResultCode.SUCCESS, feedMapper.updateById(breedFeed));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            feedMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result list(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        QueryWrapper<BreedFeed> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("create_time", calendar.getTime());
        List<FeedDataVo> feedDataVos = feedMapper.listFeed(queryWrapper);
        List<DataVo> list = new ArrayList<>();
        for (FeedDataVo feedDataVo : feedDataVos) {
            DataVo dataVo = new DataVo();
            dataVo.setCreateTime(feedDataVo.getCreateTime());
            double values = feedDataVo.getDryLeaf() + feedDataVo.getWetLeaf() + feedDataVo.getVegetable() + feedDataVo.getFeed();
            dataVo.setValues(String.valueOf(values));
            list.add(dataVo);
        }
        return new Result(ResultCode.SUCCESS, list);
    }
}
