package com.huimv.farm.musk.breed.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedLeave;
import com.huimv.farm.musk.breed.entity.BreedMuskInfo;
import com.huimv.farm.musk.breed.entity.BreedTransfer;
import com.huimv.farm.musk.breed.entity.MuskEarmark;
import com.huimv.farm.musk.breed.mapper.BreedLeaveMapper;
import com.huimv.farm.musk.breed.mapper.BreedMuskInfoMapper;
import com.huimv.farm.musk.breed.mapper.MuskEarmarkMapper;
import com.huimv.farm.musk.breed.service.IBreedLeaveService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.BaseBuilding;
import com.huimv.farm.musk.mapper.BaseBuildingMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 离场登记 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedLeaveServiceImpl extends ServiceImpl<BreedLeaveMapper, BreedLeave> implements IBreedLeaveService {

    @Autowired
    private BreedLeaveMapper leaveMapper;
    @Autowired
    private BreedMuskInfoMapper infoMapper;
    @Autowired
    private MuskEarmarkMapper earmarkMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String muskId = paramsMap.get("muskId");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedLeave> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(muskId), "musk_id", muskId);
        Page page = new Page<BreedLeave>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, leaveMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(BreedLeave breedLeave) {
        String muskId = breedLeave.getMuskId();
        QueryWrapper<BreedMuskInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("musk_id", muskId);
        BreedMuskInfo muskInfo = infoMapper.selectOne(queryWrapper);
        muskInfo.setIsLeave(1);
        infoMapper.updateById(muskInfo);
        QueryWrapper<MuskEarmark> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("earmark", muskId);
        MuskEarmark muskEarmark = earmarkMapper.selectOne(queryWrapper1);
        if (ObjectUtil.isNotEmpty(muskEarmark)) {
            muskEarmark.setIsUse(1);
            muskEarmark.setStatus(1);
            earmarkMapper.updateById(muskEarmark);
        }
        BaseBuilding building = buildingMapper.selectById(muskInfo.getUnitId());
        building.setDisabled(0);
        buildingMapper.updateById(building);
        breedLeave.setCreateTime(new Date());
        return new Result(ResultCode.SUCCESS, leaveMapper.insert(breedLeave));
    }

    @Override
    public Result edit(BreedLeave breedLeave) {
        if (breedLeave.getIsWithdraw() == 1) {
            //撤回离场
            String muskId = breedLeave.getMuskId();
            QueryWrapper<BreedMuskInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("musk_id", muskId);
            BreedMuskInfo muskInfo = infoMapper.selectOne(queryWrapper);
            muskInfo.setIsLeave(0);
            infoMapper.updateById(muskInfo);
            QueryWrapper<MuskEarmark> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("earmark", muskId);
            MuskEarmark earmark = earmarkMapper.selectOne(queryWrapper1);
            if (ObjectUtil.isNotEmpty(earmark)) {
                earmark.setIsUse(0);
                earmark.setIsUse(0);
                earmarkMapper.updateById(earmark);
            }
            BaseBuilding building = buildingMapper.selectById(muskInfo.getUnitId());
            building.setDisabled(0);
            buildingMapper.updateById(building);
        }
        return new Result(ResultCode.SUCCESS, leaveMapper.updateById(breedLeave));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            leaveMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }
}
