package com.huimv.farm.musk.breed.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.*;
import com.huimv.farm.musk.breed.entity.vo.MuskStockVo;
import com.huimv.farm.musk.breed.mapper.BreedCureMapper;
import com.huimv.farm.musk.breed.mapper.BreedInspectMapper;
import com.huimv.farm.musk.breed.mapper.BreedMuskStockMapper;
import com.huimv.farm.musk.breed.mapper.BreedVaccinationMapper;
import com.huimv.farm.musk.breed.service.IBreedMuskStockService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.DataUill;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 存栏统计 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedMuskStockServiceImpl extends ServiceImpl<BreedMuskStockMapper, BreedMuskStock> implements IBreedMuskStockService {

    @Autowired
    private BreedMuskStockMapper stockMapper;
    @Autowired
    private BreedInspectMapper inspectMapper;
    @Autowired
    private BreedCureMapper cureMapper;
    @Autowired
    private BreedVaccinationMapper vaccinationMapper;
    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BreedMuskStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<BreedMuskStock> muskStocks = stockMapper.selectList(queryWrapper);
        Integer maleNumber = 0;
        Integer femaleNumber = 0;
        Integer youngNumber = 0;
        JSONObject jsonObject = new JSONObject();
        if (muskStocks.size() != 0) {
            for (BreedMuskStock muskStock : muskStocks) {
                maleNumber = maleNumber + muskStock.getMaleNumber();
                femaleNumber = femaleNumber + muskStock.getFemaleNumber();
                youngNumber = youngNumber + muskStock.getYoungNumber();
            }
        }
        jsonObject.put("maleNumber", maleNumber);
        jsonObject.put("femaleNumber", femaleNumber);
        jsonObject.put("youngNumber", youngNumber);
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result listDetail(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BreedMuskStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<MuskStockVo> stockVos = stockMapper.listScreen(queryWrapper);
        return new Result(ResultCode.SUCCESS, stockVos);
    }

    @Override
    public Result edit(BreedMuskStock muskStock) {
        return new Result(ResultCode.SUCCESS, stockMapper.updateById(muskStock));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            stockMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listTop(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        JSONObject jsonObject = new JSONObject();

        Date monthmorning = DataUill.getTimesMonthmorning();
        QueryWrapper<BreedMuskStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("create_time", monthmorning);
        BreedMuskStock muskStock = stockMapper.selectOne(queryWrapper);
        if (ObjectUtil.isNotEmpty(muskStock)) {
            jsonObject.put("stock", muskStock.getFemaleNumber() + muskStock.getMaleNumber() + muskStock.getYoungNumber());
        } else {
            jsonObject.put("stock", 0);
        }
        Date timesmorning = DataUill.getTimesmorning();
        QueryWrapper<BreedInspect> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).ge("create_time", timesmorning);
        BreedInspect inspect = inspectMapper.selectOne(queryWrapper1);
        if (ObjectUtil.isNotEmpty(inspect)) {
            String inspectMuskId = inspect.getMuskId();
            String[] split = inspectMuskId.split(",");
            jsonObject.put("inspect", split.length);
        } else {
            jsonObject.put("inspect", 0);
        }

        QueryWrapper<BreedCure> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).ge("create_time", timesmorning);
        List<BreedCure> breedCures = cureMapper.selectList(queryWrapper2);
        if (breedCures.size() != 0) {
            Integer liveCount = 0;
            Integer deathCount = 0;
            Integer size = breedCures.size();
            for (BreedCure breedCure : breedCures) {
                if (breedCure.getCureResult() == 0) {
                    liveCount = liveCount + 1;
                } else {
                    deathCount = deathCount + 1;
                }
            }
            if (deathCount == 0) {
                jsonObject.put("cure", 100);
            } else {
                double v = deathCount.doubleValue() / size.doubleValue();
                DecimalFormat def = new DecimalFormat("0.00");
                String format = def.format(v);
                float aFloat = Float.parseFloat(format);
                float v1 = (1 - aFloat) * 100;
                jsonObject.put("cure", v1);
            }
        } else {
            jsonObject.put("cure", 100);
        }

        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result listStock(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        JSONObject jsonObject = new JSONObject();
        Date monthmorning = DataUill.getTimesMonthmorning();
        QueryWrapper<BreedMuskStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("create_time", monthmorning);
        BreedMuskStock muskStock = stockMapper.selectOne(queryWrapper);

        if (ObjectUtil.isNotEmpty(muskStock)) {
            jsonObject.put("stock", muskStock.getFemaleNumber() + muskStock.getMaleNumber() + muskStock.getYoungNumber());
        } else {
            jsonObject.put("stock", 0);
        }
        QueryWrapper<BreedCure> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).ge("create_time", monthmorning);
        List<BreedCure> breedCures = cureMapper.selectList(queryWrapper1);
        Integer cure = 0;
        if (breedCures.size() != 0) {
            for (BreedCure breedCure : breedCures) {
                if (breedCure.getCureResult() == 1) {
                    cure = cure + 1;
                }
            }
            if (ObjectUtil.isNotEmpty(muskStock)) {
                Integer i = muskStock.getFemaleNumber() + muskStock.getMaleNumber() + muskStock.getYoungNumber();
                Integer live = i - cure;
                double v = live.doubleValue() / i.doubleValue();
                DecimalFormat def = new DecimalFormat("0.00");
                String format = def.format(v);
                float aFloat = Float.parseFloat(format)*100;
                jsonObject.put("death", aFloat);
            } else {
                jsonObject.put("death", 0.00);
            }
        } else {
            jsonObject.put("death", cure);
        }

        QueryWrapper<BreedVaccination> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).ge("create_time", monthmorning);
        queryWrapper2.select("IFNULL(max(number),'0') 'number' ");
        BreedVaccination breedVaccination = vaccinationMapper.selectOne(queryWrapper2);
        if (ObjectUtil.isNotEmpty(breedVaccination)) {
            if (ObjectUtil.isNotEmpty(muskStock)) {
                Integer i = muskStock.getFemaleNumber() + muskStock.getMaleNumber() + muskStock.getYoungNumber();
                double v = breedVaccination.getNumber().doubleValue() / i.doubleValue();
                DecimalFormat def = new DecimalFormat("0.00");
                String format = def.format(v);
                float aFloat = Float.parseFloat(format)*100;
                jsonObject.put("vaccination", aFloat);
            } else {
                jsonObject.put("vaccination", 0.00);
            }
        } else {
            jsonObject.put("vaccination", 0.00);
        }
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

}
