package com.huimv.farm.musk.breed.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedMuskInfo;
import com.huimv.farm.musk.breed.entity.BreedTransfer;
import com.huimv.farm.musk.breed.entity.BreedTupping;
import com.huimv.farm.musk.breed.mapper.BreedMuskInfoMapper;
import com.huimv.farm.musk.breed.mapper.BreedTransferMapper;
import com.huimv.farm.musk.breed.service.IBreedTransferService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.BaseBuilding;
import com.huimv.farm.musk.mapper.BaseBuildingMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 转舍登记 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedTransferServiceImpl extends ServiceImpl<BreedTransferMapper, BreedTransfer> implements IBreedTransferService {

    @Autowired
    private BreedTransferMapper transferMapper;
    @Autowired
    private BreedMuskInfoMapper infoMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;
    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String muskId = paramsMap.get("muskId");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedTransfer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(muskId), "musk_id", muskId);
        Page page = new Page<BreedTransfer>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, transferMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(BreedTransfer transfer) {
        //TODO:补充林麝档案的位置信息
        String muskId = transfer.getMuskId();
        QueryWrapper<BreedMuskInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("musk_id", muskId);
        BreedMuskInfo muskInfo = infoMapper.selectOne(queryWrapper);
        muskInfo.setUnitId(transfer.getToBuildId());
        muskInfo.setUnitName(transfer.getToBuildName());
        BaseBuilding building = buildingMapper.selectById(transfer.getFromBuildId());
        building.setDisabled(0);
        buildingMapper.updateById(building);
        BaseBuilding building1 = buildingMapper.selectById(transfer.getToBuildId());
        building1.setDisabled(1);
        buildingMapper.updateById(building1);
        infoMapper.updateById(muskInfo);
        transfer.setCreateTime(new Date());
        return new Result(ResultCode.SUCCESS, transferMapper.insert(transfer));
    }

    @Override
    public Result edit(BreedTransfer transfer) {
        //TODO:补充林麝档案的位置信息
        String muskId = transfer.getMuskId();
        QueryWrapper<BreedMuskInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("musk_id", muskId);
        BreedMuskInfo muskInfo = infoMapper.selectOne(queryWrapper);
        muskInfo.setUnitId(transfer.getToBuildId());
        muskInfo.setUnitName(transfer.getToBuildName());
        BaseBuilding building = buildingMapper.selectById(transfer.getFromBuildId());
        building.setDisabled(0);
        buildingMapper.updateById(building);
        BaseBuilding building1 = buildingMapper.selectById(transfer.getToBuildId());
        building1.setDisabled(1);
        buildingMapper.updateById(building1);
        infoMapper.updateById(muskInfo);
        return new Result(ResultCode.SUCCESS, transferMapper.updateById(transfer));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            transferMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }
}
