package com.huimv.farm.musk.breed.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedCure;
import com.huimv.farm.musk.breed.entity.BreedMuskInfo;
import com.huimv.farm.musk.breed.entity.BreedTupping;
import com.huimv.farm.musk.breed.mapper.BreedMuskInfoMapper;
import com.huimv.farm.musk.breed.mapper.BreedTuppingMapper;
import com.huimv.farm.musk.breed.service.IBreedTuppingService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 配种登记 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedTuppingServiceImpl extends ServiceImpl<BreedTuppingMapper, BreedTupping> implements IBreedTuppingService {

    @Autowired
    private BreedTuppingMapper tuppingMapper;
    @Autowired
    private BreedMuskInfoMapper infoMapper;
    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String muskId = paramsMap.get("muskId");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedMuskInfo> muskInfoQueryWrapper = new QueryWrapper<>();
        muskInfoQueryWrapper.eq("farm_id", farmId).eq("musk_id", muskId);
        BreedMuskInfo muskInfo = infoMapper.selectOne(muskInfoQueryWrapper);
        QueryWrapper<BreedTupping> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        if (ObjectUtil.isNotEmpty(muskInfo)) {
            if (muskInfo.getSex() == 0) {
                queryWrapper.like(StringUtils.isNotBlank(muskId), "female_code", muskId);
            } else {
                queryWrapper.like(StringUtils.isNotBlank(muskId), "male_code", muskId);
            }
        }
        Page page = new Page<BreedTupping>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, tuppingMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(BreedTupping tupping) {
        tupping.setCreateTime(new Date());
        return new Result(ResultCode.SUCCESS, tuppingMapper.insert(tupping));
    }

    @Override
    public Result edit(BreedTupping tupping) {
        return new Result(ResultCode.SUCCESS, tuppingMapper.updateById(tupping));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            tuppingMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }
}
