package com.huimv.farm.musk.breed.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedMuskStock;
import com.huimv.farm.musk.breed.entity.BreedVaccination;
import com.huimv.farm.musk.breed.entity.vo.VaccinationVo;
import com.huimv.farm.musk.breed.mapper.BreedVaccinationMapper;
import com.huimv.farm.musk.breed.service.IBreedVaccinationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 疫苗接种 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedVaccinationServiceImpl extends ServiceImpl<BreedVaccinationMapper, BreedVaccination> implements IBreedVaccinationService {

    @Autowired
    private BreedVaccinationMapper vaccinationMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String vaccinationName = paramsMap.get("vaccinationName");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedVaccination> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(vaccinationName), "vaccination_name", vaccinationName);
        Page page = new Page<BreedVaccination>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, vaccinationMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(BreedVaccination vaccination) {
        vaccination.setCreateTime(new Date());
        return new Result(ResultCode.SUCCESS, vaccinationMapper.insert(vaccination));
    }

    @Override
    public Result edit(BreedVaccination vaccination) {
        return new Result(ResultCode.SUCCESS, vaccinationMapper.updateById(vaccination));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            vaccinationMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listCount(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        QueryWrapper<BreedVaccination> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("create_time", calendar.getTime());
        List<VaccinationVo> vaccinationVos = vaccinationMapper.listCount(queryWrapper);
        return new Result(ResultCode.SUCCESS, vaccinationVos);
    }
}
