package com.huimv.farm.musk.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.musk.common.token.TokenSign;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.SysCamera;
import com.huimv.farm.musk.entity.vo.CameraListVo;
import com.huimv.farm.musk.entity.vo.CameraListsVo;
import com.huimv.farm.musk.service.ISysCameraService;
import org.aspectj.weaver.ast.Var;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@RestController
@RequestMapping("/admin/cameraBase")
@CrossOrigin
public class SysCameraController {
    @Autowired
    private ISysCameraService cameraService;

    @PostMapping("/list")
    public Result list(@RequestBody CameraListVo cameraListVo) {
        return cameraService.listPage(cameraListVo);
    }

    @PostMapping("/listApp")
    public Result listApp(HttpServletRequest request, @RequestBody CameraListVo cameraListVo) {
        Integer userId = TokenSign.getMemberIdByJwtToken(request);
        System.out.println("用户ID：" + userId);
        return cameraService.listApp(cameraListVo, userId);
    }

    @PostMapping("/add")
    public Result add(@RequestBody SysCamera camera) {
        if (camera == null) {
            return new Result(ResultCode.FAIL, "添加失败");
        }
        return cameraService.add(camera);

    }

    @PostMapping("/listDetail")
    public Result listDetail(@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        List<SysCamera> list = cameraService.list(new QueryWrapper<SysCamera>().eq("farm_id", farmId));
        return new Result(ResultCode.SUCCESS, list);
    }

    @PostMapping("/update")
    public Result update(@RequestBody SysCamera camera) {
        return cameraService.updates(camera);

    }

    @PostMapping("/updateStatus")
    public Result updateStatus(@RequestBody Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        SysCamera sysCamera = cameraService.getById(id);
        sysCamera.setRunStatus(Math.abs(1 - sysCamera.getRunStatus()));
        cameraService.updateById(sysCamera);
        return new Result(10000, "修改成功", true);
    }

    @GetMapping("/remove")
    public Result remove(@RequestParam(value = "ids") List<Integer> ids) {
        cameraService.removeByIds(ids);
        return new Result(ResultCode.SUCCESS, "删除成功");
    }

    @PostMapping("/getCameraDetails")
    private Result getCameraDetails(HttpServletRequest request, @RequestBody CameraListsVo cameraListsVo) throws InterruptedException {
        Integer userId = TokenSign.getMemberIdByJwtToken(request);
        List list = cameraService.getCameraDetails(cameraListsVo, userId);
        return new Result(ResultCode.SUCCESS, list);
    }

    //    @GetMapping("/initCamera")
//    private Result initCamera(@RequestParam(name = "farmId") Integer farmId, HttpServletRequest request) {
//        Integer userId = TokenSign.getMemberIdByJwtToken(request);
//
//        return cameraService.initCamera(farmId, userId);
//    }
    @PostMapping("/listAll")
    public Result listAll(@RequestBody Map<String, String> paramsMap) {
        return cameraService.listAll(paramsMap);

    }

    @PostMapping("/importData")
    public Result importData(@RequestParam("file") MultipartFile file) {

        return cameraService.importData(file);

    }
}
