package com.huimv.farm.musk.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.SysCamera;
import com.huimv.farm.musk.entity.SysCameraFunction;
import com.huimv.farm.musk.entity.vo.TreeFunctionPage;
import com.huimv.farm.musk.entity.vo.TreeSysCameraFunction;
import com.huimv.farm.musk.service.ISysCameraFunctionService;
import com.huimv.farm.musk.service.ISysCameraService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 功能点 前端控制器
 * </p>
 *
 * @author author
 * @since 2024-09-11
 */
@RestController
@RequestMapping("/sys-camera-function")
@CrossOrigin
public class SysCameraFunctionController {

    @Autowired
    private ISysCameraFunctionService functionService;
    @Autowired
    private ISysCameraService cameraService;

    @RequestMapping("/listAll")
    public Result listAll(@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        List<SysCameraFunction> list = functionService.list(new QueryWrapper<SysCameraFunction>().eq("farm_id", farmId));
        List<TreeSysCameraFunction> functions = parseBizBaseArea(list);
        return new Result(ResultCode.SUCCESS, functions);
    }

    @RequestMapping("/listPage")
    public Result listPage(@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        List<SysCameraFunction> list = functionService.list(new QueryWrapper<SysCameraFunction>().eq("farm_id", farmId));
        List<TreeSysCameraFunction> functions = parseBizBaseArea(list);
        List<TreeSysCameraFunction> cameraFunctions = paginateList(functions, Integer.parseInt(pageSize), Integer.parseInt(pageNum));
        TreeFunctionPage page = new TreeFunctionPage();
        page.setTotal(cameraFunctions.size());
        page.setSize(Integer.parseInt(pageSize));
        page.setCurrent(Integer.parseInt(pageNum));
        page.setRecords(cameraFunctions);
        int i = (cameraFunctions.size() + Integer.parseInt(pageSize) - 1) / Integer.parseInt(pageSize);
        page.setPages(i);
        return new Result(ResultCode.SUCCESS, page);
    }

    @RequestMapping("/list")
    public Result list(@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        List<SysCameraFunction> list = functionService.list(new QueryWrapper<SysCameraFunction>().eq("farm_id", farmId).eq("f_type", 1));
        List<TreeSysCameraFunction> functions = parseBizBaseArea(list);
        return new Result(ResultCode.SUCCESS, functions);
    }

    @RequestMapping("/add")
    public Result add(@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String functionName = paramsMap.get("functionName");
        QueryWrapper<SysCameraFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("function_name", functionName).eq("f_type", 1);
        int count = functionService.count(queryWrapper);
        if (count != 0) {
            return new Result(10001, "功能点名称已存在！", false);
        }
        SysCameraFunction function = new SysCameraFunction();
        function.setFunctionName(functionName);
        function.setParentId(0);
        function.setFType(1);
        function.setFarmId(Integer.parseInt(farmId));
        functionService.save(function);
        return new Result(ResultCode.SUCCESS);
    }

    @RequestMapping("/edit")
    public Result edit(@RequestBody Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String functionName = paramsMap.get("functionName");
        QueryWrapper<SysCameraFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("function_name", functionName).eq("f_type", 1)
                .ne("id", id);
        int count = functionService.count(queryWrapper);
        if (count != 0) {
            return new Result(10001, "功能点名称已存在！", false);
        }
        SysCameraFunction function = functionService.getById(id);
        function.setFunctionName(functionName);
        function.setParentId(0);
        function.setFType(1);
        functionService.updateById(function);
        return new Result(ResultCode.SUCCESS);
    }

    @RequestMapping("/delete")
    public Result delete(@RequestBody Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        QueryWrapper<SysCameraFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", id);
        List<SysCameraFunction> list = functionService.list(queryWrapper);
        if (list.size() != 0) {
            for (SysCameraFunction function : list) {
                functionService.removeById(function.getId());
            }
        }
        functionService.removeById(id);

        return new Result(ResultCode.SUCCESS);
    }

    //摄像头的添加
    @RequestMapping("/addCamera")
    public Result addCamera(@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");//功能点id
        functionService.remove(new QueryWrapper<SysCameraFunction>().eq("parent_id", id));
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            SysCamera camera = cameraService.getById(s);

            SysCameraFunction function1 = new SysCameraFunction();
            function1.setFType(2);
            function1.setParentId(Integer.parseInt(id));
            function1.setFarmId(Integer.parseInt(farmId));
            function1.setCameraId(Integer.parseInt(s));
            function1.setFunctionName(camera.getCameraName());
            function1.setCameraIp(camera.getCameraIp());
            functionService.save(function1);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @RequestMapping("/deleteCamera")
    public Result editCamera(@RequestBody Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        functionService.removeById(id);
        return new Result(ResultCode.SUCCESS);
    }

    public List<TreeSysCameraFunction> parseBizBaseArea(List<SysCameraFunction> bizBaseAreas) {
        //构建需要展示的树形节点结构
        Map<Object, TreeSysCameraFunction> nodeMap = bizBaseAreas.stream().map(basePigpen -> {
            TreeSysCameraFunction baseVo = new TreeSysCameraFunction();
            baseVo.setId(basePigpen.getId());
            baseVo.setFarmId(basePigpen.getFarmId());
            baseVo.setParentId(basePigpen.getParentId());
            baseVo.setFunctionName(basePigpen.getFunctionName());
            baseVo.setCameraId(basePigpen.getCameraId());
            baseVo.setList(new ArrayList<>());
            return baseVo;
        }).collect(Collectors.toMap(TreeSysCameraFunction::getId, b -> b, (k1, k2) -> k1));
        //创建数组存父亲节点
        ArrayList<TreeSysCameraFunction> roots = new ArrayList<>();
        //构建树形
        nodeMap.values().forEach(item -> {
            Integer parentCode = item.getParentId();
            if (nodeMap.get(parentCode) == null) {
                //父节点为空，说明当前节点就已经是父节点了 将该节点存起来
                roots.add(item);
            } else {
                //父节点 不为空，说明有父节点，拿到该节点的父节点的孩子节点（就是我自己),将该节点存起来
                nodeMap.get(parentCode).getList().add(item);
            }
        });
        return roots;
    }

    public List<TreeSysCameraFunction> paginateList(List<TreeSysCameraFunction> list, int pageSize, int pageNumber) {
        List<TreeSysCameraFunction> result = new ArrayList<>();
        int startIndex = (pageNumber - 1) * pageSize;
        int endIndex = Math.min(startIndex + pageSize, list.size());
        for (int i = startIndex; i < endIndex; i++) {
            result.add(list.get(i));
        }
        return result;
    }

}
