package com.huimv.farm.musk.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.common.utils.DataUill;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.BaseBuilding;
import com.huimv.farm.musk.entity.DevEnergy;
import com.huimv.farm.musk.entity.DevEnv;
import com.huimv.farm.musk.entity.vo.DataVo;
import com.huimv.farm.musk.entity.vo.DataVo2;
import com.huimv.farm.musk.entity.vo.EnvDataVo;
import com.huimv.farm.musk.entity.vo.EnvDataVo2;
import com.huimv.farm.musk.health.entity.HealthData;
import com.huimv.farm.musk.health.mapper.HealthDataMapper;
import com.huimv.farm.musk.mapper.BaseBuildingMapper;
import com.huimv.farm.musk.mapper.DevEnvMapper;
import com.huimv.farm.musk.service.IDevEnvService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class DevEnvServiceImpl extends ServiceImpl<DevEnvMapper, DevEnv> implements IDevEnvService {

    @Autowired
    private DevEnvMapper devEnvMapper;
    @Autowired
    private BaseBuildingMapper baseBuildingMapper;
    @Autowired
    private HealthDataMapper dataMapper;



    @Override
    public Result pageDevEnv(Map<String, Object> map) {
        Integer current = (Integer) map.get("current");
        Integer size = (Integer) map.get("size");
        String farmId = (String) map.get("farmId");
        String unitId = (String) map.get("unitId");

        IPage<DevEnv> list = page(new Page<>(current, size), new QueryWrapper<DevEnv>().eq(StringUtils.isNotBlank(unitId), "unit_id", unitId).eq("farm_id", farmId));

        return new Result(ResultCode.SUCCESS, list);
    }

    @Override
    public Result listEnergy(Map<String, String> map) {
        String startDate = map.get("startDate");
        String endDate = map.get("endDate");
        String unitId = map.get("unitId");
        return new Result(ResultCode.SUCCESS, this.list(new QueryWrapper<DevEnv>().eq(StringUtils.isNotBlank(unitId), "unit_id", unitId)
                .ge(StringUtils.isNotBlank(startDate), "date_time", startDate).le(StringUtils.isNotBlank(endDate), "date_time", endDate)));

    }

    @Override
    public Result listHistory(Map<String, String> paramsMap) {

        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String timeType = paramsMap.get("timeType");

        if ("".equals(timeType) || null == timeType) {
            timeType = "1";
        }
        QueryWrapper<DevEnv> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId);
        //今日
        List<DevEnv> protEnvData = null;
        if ("1".equals(timeType)) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("date_time", timesmorning);
            protEnvData = devEnvMapper.listDataDay(queryWrapper);
        }
        //本周
        else if ("2".equals(timeType)) {
            DateTime dateTime = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("date_time", dateTime);
            protEnvData = devEnvMapper.listData(queryWrapper);
        }
        //本月
        else if ("3".equals(timeType)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("date_time", monthmorning);
            protEnvData = devEnvMapper.listData(queryWrapper);
        }
        //自定义
        else if ("4".equals(timeType)) {
            String startDate = paramsMap.get("startDate");
            String endDate = paramsMap.get("endDate");
            queryWrapper.between("date_time", startDate, endDate);
            protEnvData = devEnvMapper.listData(queryWrapper);
        }
        return new Result(ResultCode.SUCCESS, protEnvData);
    }

    @Override
    public Result listScreen(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BaseBuilding> buildingQueryWrapper = new QueryWrapper<>();
        buildingQueryWrapper.eq("farm_id", farmId).eq("id", 3);
        List<BaseBuilding> baseBuildings = baseBuildingMapper.selectList(buildingQueryWrapper);
        JSONArray jsonArray = new JSONArray();
        if (baseBuildings.size() != 0) {
            for (BaseBuilding baseBuilding : baseBuildings) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("unitName", baseBuilding.getBuildName());
                jsonObject.put("unitId", baseBuilding.getId());
                QueryWrapper<DevEnv> devEnvQueryWrapper = new QueryWrapper<>();
                devEnvQueryWrapper.eq("farm_id", farmId).orderByDesc("id").last(" limit 1").eq("unit_id", baseBuilding.getId());
                DevEnv devEnv = devEnvMapper.selectOne(devEnvQueryWrapper);
                if (ObjectUtil.isNotEmpty(devEnv)) {
                    jsonObject.put("humi", devEnv.getHumi());
                    jsonObject.put("tem", devEnv.getTem());
                    jsonObject.put("gas", devEnv.getGas());

                } else {
                    jsonObject.put("humi", "--");
                    jsonObject.put("tem", "--");
                    jsonObject.put("gas", "--");
                }
                jsonArray.add(jsonObject);
            }
        }
        return new Result(ResultCode.SUCCESS, jsonArray);
}

    @Override
    public Result listScreen2(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");//栋舍id
        Date timesmorning = DataUill.getTimesmorning();
        QueryWrapper<DevEnv> devEnvQueryWrapper = new QueryWrapper<>();
        devEnvQueryWrapper.eq("farm_id", farmId).ge("date_time", timesmorning)
                .eq("unit_id", 3).orderByAsc("date_time");
        return new Result(ResultCode.SUCCESS, devEnvMapper.selectList(devEnvQueryWrapper));
    }

    @Override
    public Result listByType(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        List<EnvDataVo2> list = new ArrayList<>();

        QueryWrapper<BaseBuilding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("f_type", 1);
        List<BaseBuilding> buildings = baseBuildingMapper.selectList(queryWrapper1);
        if (buildings.size() != 0) {
            for (BaseBuilding building : buildings) {
                EnvDataVo2 protDataVo2 = new EnvDataVo2();
                protDataVo2.setUnitId(building.getId());
                protDataVo2.setUnitName(building.getBuildName());
                QueryWrapper<DevEnv> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", farmId).eq("unit_id", building.getId()).ge("date_time", timesmorning);
                List<EnvDataVo> protDataVos = devEnvMapper.listScreen(queryWrapper);
                protDataVo2.setList(protDataVos);
                if (protDataVos.size() != 0) {
                    EnvDataVo protDataVo = protDataVos.get(protDataVos.size() - 1);
                    protDataVo2.setGas(protDataVo.getGas());
                    protDataVo2.setHumi(protDataVo.getHumi());
                    protDataVo2.setTem(protDataVo.getTem());
                } else {
                    protDataVo2.setGas("0");
                    protDataVo2.setHumi("0");
                    protDataVo2.setTem("0");
                }
                list.add(protDataVo2);
            }
        }
        return new Result(ResultCode.SUCCESS, list);
    }

    @Override
    public Result listAll(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        QueryWrapper<DevEnv> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("date_time", timesmorning);
        List<DataVo> envTemps = devEnvMapper.listTem(queryWrapper);

        QueryWrapper<HealthData> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).ge("create_time", timesmorning);
        List<DataVo> healths = dataMapper.listTemp(queryWrapper1);

        List<DataVo2> list = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        for (int i = 0; i < 24; i++) {
            DataVo2 dataVo2 = new DataVo2();
            if (i < 10) {
                dataVo2.setTime(sdf.format(timesmorning)+" 0" + i+":00:00");
            } else {
                dataVo2.setTime(sdf.format(timesmorning)+" "+i+":00:00");
            }
            if (envTemps.size() != 0) {
                for (DataVo envTemp : envTemps) {
                    if (envTemp.getCreateTime().getHours() == i) {
                        dataVo2.setEnvTemp(envTemp.getValues());
                    }
                }
            }
            if (healths.size() != 0) {
                for (DataVo health : healths) {
                    if (health.getCreateTime().getHours() == i) {
                        dataVo2.setHealthTemp(health.getValues());
                    }
                }
            }
            list.add(dataVo2);
        }
        return new Result(ResultCode.SUCCESS, list);
    }
}
