package com.huimv.farm.musk.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.ExcelImportSheet;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.SysAccountMultilevel;
import com.huimv.farm.musk.entity.SysCamera;
import com.huimv.farm.musk.entity.SysCameraArea;
import com.huimv.farm.musk.entity.SysCameraBrand;
import com.huimv.farm.musk.entity.vo.*;
import com.huimv.farm.musk.mapper.SysAccountMultilevelMapper;
import com.huimv.farm.musk.mapper.SysCameraAreaMapper;
import com.huimv.farm.musk.mapper.SysCameraBrandMapper;
import com.huimv.farm.musk.mapper.SysCameraMapper;
import com.huimv.farm.musk.service.ISysCameraService;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class SysCameraServiceImpl extends ServiceImpl<SysCameraMapper, SysCamera> implements ISysCameraService {

    @Autowired
    private SysCameraMapper cameraBaseMapper;
    @Autowired
    private SysAccountMultilevelMapper sysAccountMultilevelMapper;
    @Autowired
    private SysCameraAreaMapper cameraAreaMapper;
    @Autowired
    private SysCameraBrandMapper cameraBrandMapper;

    @Autowired
    private SysCameraMapper cameraMapper;

//    @Autowired
//    private BaseScreenCameraAccountMapper cameraAccountMapper;

    @Override
    public Result listPage(CameraListVo cameraListVo) {
        Integer current = cameraListVo.getCurrent();
        String strchStr = cameraListVo.getStrchStr();
        Integer size = cameraListVo.getSize();
        Integer areaId = cameraListVo.getAreaId();
        Integer brandId = cameraListVo.getBrandId();
        Integer sort = cameraListVo.getSort();
        Integer farmId = cameraListVo.getFarmId();
        Integer isOnline = cameraListVo.getIsOnline();

        QueryWrapper<SysCamera> wrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(strchStr)) {
            wrapper.and(i -> i.like("camera_name", strchStr)
                    .or().like("camera_ip", strchStr));
        }
//        if (userId != 1){
//            SysAccountMultilevel sysAccountMultilevel = sysAccountMultilevelMapper.selectById(userId);
//            String cameraIds = sysAccountMultilevel.getCameraIds();
//            List<String> list = new ArrayList();
//            if (StringUtils.isNotBlank(cameraIds)){
//                list  = Arrays.asList(cameraIds);
//            }
//            wrapper.in("id",list);
//        }

        if (farmId != null) {
            wrapper.eq("farm_id", farmId);
        }

        if (areaId != null) {
            wrapper.eq("area_id", areaId);
        }

        if (brandId != null) {
            wrapper.eq("brand_id", brandId);
        }

        if (isOnline != null) {
            wrapper.eq("is_online", isOnline);
        }

        if (sort == null) {
            wrapper.orderByDesc("id");
        } else if (sort == 1) {
            wrapper.orderByDesc("sort");
        } else {
            wrapper.orderByAsc("sort");
        }
//        wrapper.orderByDesc("id");
        return new Result(ResultCode.SUCCESS, page(new Page<>(current, size), wrapper));
    }


    @Override
    public List getCameraDetails(CameraListsVo cameraListsVo,Integer userId) throws InterruptedException {
        List<Integer> cameraIds = cameraListsVo.getCameraIds();
        CameraListVo cameraListVo = new CameraListsVo();
        cameraListVo.setCurrent(1);
        cameraListVo.setSize(10000);
        cameraListVo.setStrchStr(cameraListsVo.getStrchStr());
        cameraListVo.setAreaId(cameraListsVo.getAreaId());
        cameraListVo.setBrandId(cameraListsVo.getBrandId());
        cameraListVo.setSort(cameraListsVo.getSort());

        if (cameraIds == null || cameraIds.size() == 0) {
            return null;
        }
        List cameraDetailsList = new ArrayList();
        IPage<SysCamera> page = (IPage<SysCamera>) this.listPage(cameraListVo).getData();
        List<SysCamera> list = page.getRecords();
        int i = 0;
        final String farmAppPort = "8081";
        final String farmVideoApp = "/appleVideo/play?";
        final String hlsMediaPort = "85";
        for (Integer cameraId : cameraIds) {
            for (SysCamera camera : list) {
                if (cameraId.equals(camera.getId())) {

                    String cameraAccount = camera.getAccount();
                    String cameraPassword = camera.getPassword();
                    String camerIp = camera.getCameraIp();
                    String cameraName = camera.getCameraName();
                    Integer areaId = camera.getAreaId();
                    String url = cameraAreaMapper.selectById(areaId).getPublicUrl();
                    String rtsp;
                    SysCameraBrand cameraBrand = cameraBrandMapper.selectById(camera.getBrandId());
                    Integer type = cameraBrand.getStreamType();
                    if (type == 0) {
                        rtsp = cameraBrand.getMainStream();
                    } else {
                        rtsp = cameraBrand.getAssistStream();
                    }
                    CameraVo cameraVo = new CameraVo();
                    cameraVo.setId(camera.getId());
                    cameraVo.setAreaId(areaId);

                    cameraVo.setWsUrl("ws://" + url + "/camera_relay?tcpaddr=" + cameraAccount + "%3A" + cameraPassword + "%40" + camerIp /*+ "/h265/ch1/main/av_stream"*/);
                    cameraVo.setRtspUrl("rtsp://" + cameraAccount + ":" + cameraPassword + "@" + camerIp + rtsp);

                    cameraVo.setCameraName(cameraName);
                    cameraVo.setHttpUrl(handleHlsHttpUrl(url,cameraAccount,cameraPassword,camerIp,farmAppPort,farmVideoApp,hlsMediaPort));
                    if (i > 0) {
                        cameraVo.setLastId(list.get(i - 1).getId());
                    }
                    if (i < list.size() - 1) {
                        cameraVo.setNextId(list.get(i + 1).getId());
                    }
                    cameraDetailsList.add(cameraVo);
                }
            }
            i++;
        }
        return cameraDetailsList;
    }

    //构造HlsHttp地址
    private String handleHlsHttpUrl(String url, String cameraAccount, String cameraPassword, String camerIp, String farmAppPort, String farmVideoApp, String hlsMediaPort) {
        url = url.substring(0, url.indexOf(":"));
        url = "http://" + url;
        //return url+":"+farmAppPort+farmVideoApp+"url="+cameraAccount + ":" + cameraPassword + "@" + camerIp+"&name="+"&dt=3000&targetUrl="+url+":"+hlsMediaPort;
        return url + ":" + farmAppPort + farmVideoApp + "url=" + cameraAccount + ":" + cameraPassword + "@" + camerIp + "&dt=3000&targetUrl=" + url + ":" + hlsMediaPort;
    }

    @Override
    public Result add(SysCamera camera) {
        Integer areaId = camera.getAreaId();
        Integer brandId = camera.getBrandId();

        int brandName = this.count(new QueryWrapper<SysCamera>().eq("camera_name", camera.getCameraName()).eq("farm_id", camera.getFarmId()));
        if (brandName > 0) {
            return new Result(10001, "摄像头名称重复", false);
        }
        if (areaId != null) {
            camera.setAreaName(cameraAreaMapper.selectById(areaId).getAreaName());
        }
        if (brandId != null) {
            camera.setBrandName(cameraBrandMapper.selectById(brandId).getBrandName());
        }
        this.save(camera);

        return new Result(10000, "添加成功", true);
    }

    @Override
    public Result updates(SysCamera camera) {
        Integer areaId = camera.getAreaId();
        Integer brandId = camera.getBrandId();
        this.updateById(camera);
        int brandName = this.count(new QueryWrapper<SysCamera>().eq("camera_name", camera.getCameraName()));
        if (brandName > 1) {
            return new Result(10001, "摄像头名称重复", false);
        }
        if (areaId != null) {
            camera.setAreaName(cameraAreaMapper.selectById(areaId).getAreaName());
        }
        if (brandId != null) {
            camera.setBrandName(cameraBrandMapper.selectById(brandId).getBrandName());
        }
        this.updateById(camera);
        return new Result(10000, "修改成功", true);
    }

    @Override
    public List<String> listIds() {
        return cameraBaseMapper.listIds();
    }

    @Override
    public Result listApp(CameraListVo cameraListVo, Integer userId) {
        List endList = new ArrayList();
        List<SysCameraArea> cameraAreaList = cameraAreaMapper.selectList(new QueryWrapper<SysCameraArea>().eq("farm_id", cameraListVo.getFarmId())
                .eq("run_status",1).orderByAsc("sort"));
        SysAccountMultilevel sysAccountMultilevel = sysAccountMultilevelMapper.selectById(userId);
        String cameraIds = sysAccountMultilevel.getCameraIds();
        String[] split = cameraIds.split(",");
        if (userId != 1){
            for (SysCameraArea cameraArea : cameraAreaList) {
                List<SysCamera> sysCameras = cameraMapper.selectList(new QueryWrapper<SysCamera>().eq("area_id", cameraArea.getId())
                        .eq("run_status", 1).in("id",split).orderByAsc("sort"));
                Map map = new HashMap();
                map.put("cameraName", cameraArea.getAreaName());
                map.put("id", cameraArea.getId());
                if (ObjectUtils.isNotEmpty(sysCameras)) {
                    map.put("cameraList", sysCameras);
                } else {
                    map.put("cameraList", "");
                }
                endList.add(map);
            }
            return new Result(ResultCode.SUCCESS, endList);
        }else {
            for (SysCameraArea cameraArea : cameraAreaList) {
                Map map = new HashMap();
                map.put("cameraName", cameraArea.getAreaName());
                map.put("id", cameraArea.getId());
                map.put("cameraList", cameraMapper.selectList(new QueryWrapper<SysCamera>().eq("area_id", cameraArea.getId())
                        .eq("run_status",1).orderByAsc("sort")));
                endList.add(map);
            }
            return new Result(ResultCode.SUCCESS, endList);
        }

    }

    @Override
    public Result listAll(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        List<SysCamera> cameras = cameraMapper.selectList(new QueryWrapper<SysCamera>().eq("farm_id", farmId));
        JSONObject jsonObject = new JSONObject();
        Integer count = 0;
        Integer offCount = 0;
        Integer onCount = 0;
        count = cameras.size();
        if (cameras.size() != 0) {
            for (SysCamera camera : cameras) {
                if (camera.getIsOnline()==1) {
                    onCount = onCount + 1;
                } else {
                    offCount = offCount + 1;
                }
            }
        }
        jsonObject.put("count", count);
        jsonObject.put("offCount", count);
        jsonObject.put("onCount", count);
        return new Result(ResultCode.SUCCESS,jsonObject);
    }

    @Transactional
    @Override
    public Result importData( MultipartFile file) {

        Map<String, String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("摄像头名称", "cameraName");
        map.put("地址", "cameraIp");

        try (
                //这里面的对象会自动关闭
                InputStream in = file.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in, file.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<ImportDataVo> list = ExcelImportSheet.getListFromExcel(sheet, ImportDataVo.class, map);

            for (ImportDataVo importDataVo : list) {
                SysCamera sysCamera = new SysCamera();
                sysCamera.setCameraName(importDataVo.getCameraName());
                sysCamera.setCameraIp(importDataVo.getCameraIp());
                sysCamera.setAccount("admin");
                sysCamera.setPassword("adminjhd123");
                sysCamera.setAreaName("B区");
                sysCamera.setAreaId(4);
                sysCamera.setAttention(0);
                sysCamera.setBrandName("大华");
                sysCamera.setBrandId(1);
                sysCamera.setRunStatus(1);
                sysCamera.setSort(1);
                sysCamera.setFarmId(330110007);
                sysCamera.setIsOnline(1);
                cameraMapper.insert(sysCamera);
            }
        }catch (Exception e){
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        }
        return new Result(ResultCode.SUCCESS);
    }

//    @Override
//    public Result initCamera(Integer farmId, Integer userId) {
//        BaseScreenCameraAccount cameraAccount = cameraAccountMapper.selectOne(new QueryWrapper<BaseScreenCameraAccount>().eq("farm_id", farmId).eq("account_id", userId));
//        Integer cameraId;
//        if (ObjectUtil.isEmpty(cameraAccount)) {
//            cameraId = this.getOne(new QueryWrapper<SysCamera>().eq("run_status", 1).eq("farm_id", farmId).last("limit 1")).getId();
//        } else {
//            cameraId = cameraAccount.getCameraId();
//        }
//
//
//        SysCamera camera = this.getById(cameraId);
//        if (camera.getRunStatus() == 0) {
//            return new Result(10001, "该摄像头已经被禁用", false);
//        }
//
//        return new Result(ResultCode.SUCCESS, getCameraVo(camera));
//    }

    public CameraVo getCameraVo(SysCamera camera) {

        String cameraAccount = camera.getAccount();
        String cameraPassword = camera.getPassword();
        String camerIp = camera.getCameraIp();
        String cameraName = camera.getCameraName();
        Integer areaId = camera.getAreaId();
        String url = cameraAreaMapper.selectById(areaId).getPublicUrl();

        String rtsp;
        SysCameraBrand cameraBrand = cameraBrandMapper.selectById(camera.getBrandId());
        Integer type = cameraBrand.getStreamType();
        if (type == 0) {
            rtsp = cameraBrand.getMainStream();
        } else {
            rtsp = cameraBrand.getAssistStream();
        }


        CameraVo cameraVo = new CameraVo();
        cameraVo.setAreaId(areaId);

        cameraVo.setWsUrl("ws://" + url + "/camera_relay?tcpaddr=" + cameraAccount + "%3A" + cameraPassword + "%40" + camerIp);
        cameraVo.setRtspUrl("rtsp://" + cameraAccount + ":" + cameraPassword + "@" + camerIp + rtsp);
        cameraVo.setCameraName(cameraName);
        return cameraVo;
    }


}
